/*   This file is part of Xsltop.
*
*    Xsltop is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation.
*
*    Xsltop is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY.  See the
*    GNU General Public License for more details.
*
*    See <http://www.gnu.org/licenses/>.
*/
package xsltop.xml;

import javax.xml.namespace.NamespaceContext;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * The Class XPathHelper.
 */
public class XPathHelper {
	
	/** The namespace. */
	private static NamespaceContext ns;
	
	/**
	 * Sets the namespace context.
	 * 
	 * @param pNamespace the new namespace context
	 */
	public static void setNamespaceContext(NamespaceContext pNamespace) {
		XPathHelper.ns = pNamespace;
	}
	
	/**
	 * Instantiates a new x path helper.
	 */
	private XPathHelper() {
		super();
	}
	
	/**
	 * Eval int.
	 * 
	 * @param pDoc the document
	 * @param pExpression the expression
	 * 
	 * @return the int
	 * 
	 * @throws XsltopException the xsltop exception
	 */
	public static int evalInt(final Document pDoc, final String pExpression)
	throws XsltopException {
		int vEval;
		try {
            final XPath xpath = XPathFactory.newInstance().newXPath();                        
            final Double xobject = (Double) xpath
            						.compile(pExpression)
            						.evaluate(pDoc, XPathConstants.NUMBER);
            vEval = xobject.intValue();            
        } catch(XPathExpressionException e) {
            throw new XsltopException(e);            
        }
		return vEval;
	}
	
	/**
	 * Eval list.
	 * 
	 * @param pDoc the document
	 * @param pExpression the expression
	 * 
	 * @return the node list
	 * 
	 * @throws XsltopException the xsltop exception
	 */
	public static NodeList evalList(final Document pDoc,
			final String pExpression) throws XsltopException {
		NodeList vNodes;		
		try {

			final XPath xpath = XPathFactory.newInstance().newXPath();
            if (ns != null) {
            	xpath.setNamespaceContext(ns);
            }
            vNodes = (NodeList) xpath
            				.compile(pExpression)
            				.evaluate(pDoc, XPathConstants.NODESET);
        } catch(XPathExpressionException e) {
            throw new XsltopException(e);            
        }
		return vNodes;
	}
	
	/**
	 * Eval.
	 * 
	 * @param document the document
	 * @param expression the expression
	 * 
	 * @return the node
	 * 
	 * @throws XsltopException the xsltop exception
	 */
	public static Node eval(final Document document, final String expression)
	throws XsltopException {
		Node vNode;
		try{
            final XPath xpath = XPathFactory.newInstance().newXPath();            
            vNode = (Node) xpath
            				.compile(expression)
            				.evaluate(document, XPathConstants.NODE);
        } catch(XPathExpressionException e) {
            throw new XsltopException(e);            
        }
		return vNode;
	}
}
